package com.arms.api.alm.analyis.service;

import com.arms.api.analysis.resource.dto.ResourceAggrDTO;
import com.arms.api.analysis.resource.service.AnalysisResource;
import com.arms.api.issue.almapi.model.entity.AlmIssueEntity;
import com.arms.api.util.model.dto.PdServiceAndIsReqDTO;
import com.arms.egovframework.javaservice.esframework.esquery.EsQuery;
import com.arms.egovframework.javaservice.esframework.esquery.EsQueryBuilder;
import com.arms.egovframework.javaservice.esframework.esquery.filter.ExistsQueryFilter;
import com.arms.egovframework.javaservice.esframework.esquery.filter.TermsQueryFilter;
import com.arms.egovframework.javaservice.esframework.esquery.must.TermQueryMust;
import com.arms.egovframework.javaservice.esframework.model.dto.집계_하위_필드;
import com.arms.egovframework.javaservice.esframework.model.vo.v2.DocumentAggregations;
import com.arms.egovframework.javaservice.esframework.model.vo.v2.DocumentBucket;
import com.arms.egovframework.javaservice.esframework.repository.EsCommonRepositoryWrapper;
import lombok.extern.slf4j.Slf4j;
import org.junit.jupiter.api.Test;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.context.SpringBootTest;
import org.springframework.test.context.ActiveProfiles;

import java.util.List;

import static com.arms.egovframework.javaservice.esframework.factory.creator.중첩_집계_쿼리_생성기.포괄V2;

@ActiveProfiles("dev")
@SpringBootTest
@Slf4j
public class AnalysisResourceTest extends EsCommonRepositoryWrapper<AlmIssueEntity> {

    @Autowired
    AnalysisResource analysisResource;

    @Test
    public void test(){

        PdServiceAndIsReqDTO pdServiceAndIsReqDTO = new PdServiceAndIsReqDTO();
        pdServiceAndIsReqDTO.setIsReq(false);
        pdServiceAndIsReqDTO.setPdServiceId(27L);
        pdServiceAndIsReqDTO.setPdServiceVersions(List.of(84L,85L,86L));

        ResourceAggrDTO resourceAggrDTO = new ResourceAggrDTO();
        resourceAggrDTO.setPdServiceAndIsReq(pdServiceAndIsReqDTO);
        resourceAggrDTO.set메인_그룹_필드("pdServiceVersions");
        resourceAggrDTO.set컨텐츠_보기_여부(false);
        resourceAggrDTO.set집계_하위_필드들(
                List.of(
                        집계_하위_필드.builder()
                                .하위_필드명_별칭("assigneeID")
                                .하위_필드명("assignee.assignee_accountId.keyword")
                                .크기(5)
                                .build(),
                        집계_하위_필드.builder()
                                .하위_필드명_별칭("assigneeName")
                                .하위_필드명("assignee.assignee_displayName.keyword")
                                .build(),
                        집계_하위_필드.builder()
                                .하위_필드명_별칭("assigneeEmail")
                                .하위_필드명("assignee.assignee_emailAddress.keyword")
                                .build()
                )
        );

        EsQuery notReqIssueQuery = esQueryWithAssignee(resourceAggrDTO, Boolean.FALSE);
        resourceAggrDTO.setEsQuery(notReqIssueQuery);

        DocumentAggregations documentAggregations = this.aggregateRecentDocs(resourceAggrDTO);

        System.out.println("--------------------- -----------------------");
        System.out.println(documentAggregations.getTotalHits()); // 11
        System.out.println(documentAggregations.getDocSumCount()); // 19
        System.out.println("--------------------- -----------------------");
        System.out.println("--------------------- -----------------------");
        List<DocumentBucket> documentBuckets = documentAggregations.deepestSubList();
        for (DocumentBucket documentBucket : documentBuckets) {
            System.out.println(documentBucket.getGroupKey());
            System.out.println(documentBucket.getDocFieldName("group_by_pdServiceVersions"));
            System.out.println(documentBucket.getDocFieldName("assigneeID"));
            System.out.println(documentBucket.getDocFieldName("assigneeName"));
            System.out.println(documentBucket.getDocFieldName("assigneeEmail"));
            System.out.println(documentBucket.getDocCount());
        }
        System.out.println("--------------------- -----------------------");

        // documentCount 와 aggs의 갯수가 불일치 할 수 있음 (한 이슈에 버전이 여러개일 수 있으니까)
        // 필요한 것 :: Long versionId 에 List<Worker> 이 들어가야함.

    }

    private EsQuery esQueryWithAssignee(ResourceAggrDTO resourceAggrDTO, Boolean isReq) {
        EsQuery esQuery = new EsQueryBuilder()
                .bool(
                        new TermQueryMust("pdServiceId",resourceAggrDTO.getPdServiceLink()),
                        new TermsQueryFilter("pdServiceVersions",resourceAggrDTO.getPdServiceVersionLinks()),
                        new TermQueryMust("isReq", isReq),
                        new ExistsQueryFilter("assignee")
                );
        return esQuery;
    }
}
